/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtWidgets>
#include <QDebug>
#include "codeeditor.h"
#include <QFile>

//![constructor]

CodeEditor::CodeEditor(QWidget *parent) : QPlainTextEdit(parent)
{
    lineNumberArea = new LineNumberArea(this);

    connect(this, SIGNAL(blockCountChanged(int)), this, SLOT(updateLineNumberAreaWidth(int)));
    connect(this, SIGNAL(updateRequest(QRect,int)), this, SLOT(updateLineNumberArea(QRect,int)));
    connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(highlightCurrentLine()));


    new GCodeSyntaxHighlighter(document());

    updateLineNumberAreaWidth(0);
    highlightCurrentLine();
}

int CodeEditor::lineNumberAreaWidth()
{
    int digits = 1;
    int max = qMax(1, blockCount());
    while (max >= 10) {
        max /= 10;
        ++digits;
    }

    int space = 20 + fontMetrics().width(QLatin1Char('9')) * digits;

    return space;
}

void CodeEditor::updateLineNumberAreaWidth(int /* newBlockCount */)
{
    setViewportMargins(lineNumberAreaWidth(), 0, 0, 0);
}

void CodeEditor::updateLineNumberArea(const QRect &rect, int dy)
{
    if (dy)
        lineNumberArea->scroll(0, dy);
    else
        lineNumberArea->update(0, rect.y(), lineNumberArea->width(), rect.height());

    if (rect.contains(viewport()->rect()))
        updateLineNumberAreaWidth(0);
}

void CodeEditor::resizeEvent(QResizeEvent *e)
{
    QPlainTextEdit::resizeEvent(e);

    QRect cr = contentsRect();
    lineNumberArea->setGeometry(QRect(cr.left(), cr.top(), lineNumberAreaWidth(), cr.height()));
}

void CodeEditor::highlightCurrentLine()
{
    QList<QTextEdit::ExtraSelection> extraSelections;

    if (!isReadOnly()) {
        QTextEdit::ExtraSelection selection;

        QColor lineColor = QColor(Qt::yellow).lighter(160);

        selection.format.setBackground(lineColor);
        selection.format.setProperty(QTextFormat::FullWidthSelection, true);
        selection.cursor = textCursor();
        selection.cursor.clearSelection();
        extraSelections.append(selection);
    }
    setExtraSelections(extraSelections);
}

void CodeEditor::lineNumberAreaPaintEvent(QPaintEvent *event)
{
    QPainter painter(lineNumberArea);
    painter.fillRect(event->rect(), QColor(240,240,240));

    QTextBlock block = firstVisibleBlock();
    int blockNumber = block.blockNumber();
    int top = (int) blockBoundingGeometry(block).translated(contentOffset()).top();
    int bottom = top + (int) blockBoundingRect(block).height();

    while (block.isValid() && top <= event->rect().bottom()) {
        if (block.isVisible() && bottom >= event->rect().top()) {
            QString number = QString::number(blockNumber + 1);
            painter.setPen(Qt::darkGray);
            painter.drawText(0, top, lineNumberArea->width()-10, fontMetrics().height(),
                             Qt::AlignRight, number);
        }

        block = block.next();
        top = bottom;
        bottom = top + (int) blockBoundingRect(block).height();
        ++blockNumber;
    }
}

void CodeEditor::dropEvent(QDropEvent *event)
{
    qDebug()<<"Drop Event in CodeEditor class";
    QPlainTextEdit::dropEvent(event); //seems to be necessary  to call baseclass method to make it work poperly
    if(event->mimeData()->hasUrls()){
            QList<QUrl> urls = event->mimeData()->urls();

            if (urls.isEmpty())return;

            QString fileName = urls.first().toLocalFile();

            if (fileName.isEmpty()) return;

            if(!fileName.endsWith(".nc",Qt::CaseInsensitive)
               && !fileName.endsWith(".cnc",Qt::CaseInsensitive)
                && !fileName.endsWith(".gcode",Qt::CaseInsensitive)
                    ){
                return;
            }
            QFile LoadFile(fileName);
            if(!LoadFile.open(QIODevice::ReadOnly)){return;}
            clear();
            appendPlainText(LoadFile.readAll());
    }
}


GCodeSyntaxHighlighter::GCodeSyntaxHighlighter(QTextDocument* document) :
    QSyntaxHighlighter(document)
{

}

void GCodeSyntaxHighlighter::highlightBlock(const QString &text)
{
    QTextCharFormat ClassFormat;
    QRegExp Expression;

    ClassFormat.setFontWeight(QFont::Bold);

    QStringList RegExpList;
    QList<QColor> ColorList;

    RegExpList<<"[nN][0-9]*";
    ColorList<<Qt::gray;

    RegExpList<<"[mM][0-9]*";
    ColorList<<Qt::red;

    RegExpList<<"[gG][0-9]*";
    ColorList<<Qt::darkBlue;

    RegExpList<<"[xX]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkGreen;

    RegExpList<<"[yY]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkCyan;

    RegExpList<<"[zZ]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::blue;

    RegExpList<<"[iI]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkYellow;

    RegExpList<<"[jJ]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkYellow;

    RegExpList<<"[kK]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkYellow;

    RegExpList<<"[rR]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkGray;

    RegExpList<<"[sS]-?[0-9]*\\.?[0-9]*";
    ColorList<<Qt::darkMagenta;

    RegExpList<<"[fF]-?[0-9]*\\.?[0-9]*";
    ColorList<<QColor(255,127,35);

    RegExpList<<"[(][^)]*[)]";
    ColorList<<Qt::black;

    currentBlock().blockNumber();

    for(int k = 0; k < RegExpList.size(); k++){
        ClassFormat.setForeground(ColorList.value(k,Qt::red));
        Expression =  QRegExp(RegExpList.value(k,""));
        int index = text.indexOf(Expression);
        while (index >= 0) {
            int length = Expression.matchedLength();
            setFormat(index, length, ClassFormat);
            index = text.indexOf(Expression, index + length);
        }
    }
}
