// OpenGLWidget.cpp

#include <QtWidgets>
#include <QtOpenGL>

#include "openglwidget.h"

OpenGLWidget::OpenGLWidget(QWidget *parent)
    : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{
    xRot = yRot =  zRot = 0;
    xMove = yMove = zMove = 0;
    Zoom = 1000;
    WindowRatio = 1;

    DoDrawOrigin = DoDrawBounding = true;

    PreviewDataPtr = 0;
}

OpenGLWidget::~OpenGLWidget()
{
}

void OpenGLWidget::setOriginDrawing(bool draw)
{
    if(draw ==  DoDrawOrigin) return;
    DoDrawOrigin = draw;
    updateGL();
}

void OpenGLWidget::setBoundingDrawing(bool draw)
{
    if(draw ==  DoDrawBounding) return;
    DoDrawBounding = draw;
    updateGL();
}

void OpenGLWidget::setViewFromAbove()
{
    xRot = yRot =  zRot = 0;
    zMove = 0;

    Zoom = 1000;
    xMove = ((min_x+max_x)/2)* (-10.0);
    zMove = ((min_y+max_y)/2)* (-10.0);

    Zoom *= qMin(3200.0/(max_x - min_x), (3200.0/(max_y - min_y))*WindowRatio);

    updateGL();
}

QSize OpenGLWidget::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize OpenGLWidget::sizeHint() const
{
    return QSize(400, 400);
}


void OpenGLWidget::initializeGL()
{
  //  qglClearColor(QColor(Qt::yellow).lighter(180));
    qglClearColor(QColor(Qt::darkGray).darker(150));

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glShadeModel(GL_SMOOTH);
}

void OpenGLWidget::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    glScalef(1.0+0.1*Zoom, 1.0+0.1*Zoom,1.0+0.1*Zoom);
    glTranslatef(xMove*0.1, zMove*0.1, -10.0);
    glRotatef(xRot / 16.0, 1.0, 0.0, 0.0);
    glRotatef(yRot / 16.0, 0.0, 1.0, 0.0);
    glRotatef(zRot / 16.0, 0.0, 0.0, 1.0);
    draw();
}

void OpenGLWidget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    WindowRatio = (float)height/(float)width;
    glOrtho(-200000.0, +200000.0, -200000.0*WindowRatio, +200000.0*WindowRatio, -2000000.0, 2000000.0);
    glMatrixMode(GL_MODELVIEW);
}

void OpenGLWidget::mousePressEvent(QMouseEvent *event)
{
    lastPos = event->pos();
}

void OpenGLWidget::mouseMoveEvent(QMouseEvent *event)
{
    int dx = event->x() - lastPos.x();
    int dy = event->y() - lastPos.y();

    if (event->buttons() & Qt::LeftButton) {
        zRot += 8 * dx;
        xRot += 8 * dy;
    } else if (event->buttons() & Qt::RightButton) {
        zMove -= 8 * dy *  10000.0/(float)Zoom;
        xMove += 8 * dx *  10000.0/(float)Zoom;
    }

    zMove = qBound(-10000, zMove, 10000);
    xMove = qBound(-18000, xMove, 18000);
    xRot = qBound(-2880, xRot, 0);

    lastPos = event->pos();
    updateGL();
}

void OpenGLWidget::wheelEvent(QWheelEvent *event)
{
    if(event->delta() >0){
        Zoom = qMin(100000.0, Zoom*1.35);
    }else{
        Zoom = qMax(1000.0, Zoom/1.35);
    }

    updateGL();
}

void OpenGLWidget::draw()
{

    min_x = 0;
    max_x = 0;
    min_y = 0;
    max_y = 0;
    min_z = 0;
    max_z = 0;

    if(PreviewDataPtr == 0){return;}
    if(PreviewDataPtr->size() < 2){
        PreviewDataPtr->append(QVector4D(0,0,0,0));
        PreviewDataPtr->append(QVector4D(0,0,0,0));
    }

    qDebug()<<"Draw";
    bool CurrentIsG0 = PreviewDataPtr->at(0).w() < 0.5;

        qglColor(CurrentIsG0? (Qt::red) : (Qt::green));

        glBegin(GL_LINE_STRIP);
            for(int i = 0; i <PreviewDataPtr->size(); i++){

                bool newIsG0 = PreviewDataPtr->at(i).w() < 0.5;

                if(CurrentIsG0 != newIsG0){
                    CurrentIsG0 = newIsG0;
                    glEnd();
                     qglColor(CurrentIsG0? (Qt::red) : (Qt::green));
                    glBegin(GL_LINE_STRIP);
                    glVertex3f(PreviewDataPtr->at(i-1).x(),PreviewDataPtr->at(i-1).y(),PreviewDataPtr->at(i-1).z());
                }


                glVertex3f(PreviewDataPtr->at(i).x(),PreviewDataPtr->at(i).y(),PreviewDataPtr->at(i).z());

                min_x = qMin(min_x, PreviewDataPtr->at(i).x());
                min_y = qMin(min_y, PreviewDataPtr->at(i).y());
                min_z = qMin(min_z, PreviewDataPtr->at(i).z());

                max_x = qMax(max_x, PreviewDataPtr->at(i).x());
                max_y = qMax(max_y, PreviewDataPtr->at(i).y());
                max_z = qMax(max_z, PreviewDataPtr->at(i).z());
            }
        glEnd();



    max_z = 0;
    min_x -= 10.0;
    min_y -= 10.0;
    max_x += 10.0;
    max_y += 10.0;

    if(NumberLCDsPtrs[0]){NumberLCDsPtrs[0]->display(max_x - min_x);}
    if(NumberLCDsPtrs[1]){NumberLCDsPtrs[1]->display(max_y - min_y);}
    if(NumberLCDsPtrs[2]){NumberLCDsPtrs[2]->display(max_z - min_z);}

    if(DoDrawBounding){
        qglColor(Qt::cyan);
        glBegin(GL_LINE_STRIP);
            glVertex3f(min_x,min_y,min_z);
            glVertex3f(min_x,min_y,max_z);
            glVertex3f(max_x,min_y,max_z);
            glVertex3f(max_x,min_y,min_z);
            glVertex3f(min_x,min_y,min_z);

            glVertex3f(min_x,max_y,min_z);
            glVertex3f(min_x,max_y,0.0);
            glVertex3f(max_x,max_y,0.0);
            glVertex3f(max_x,max_y,min_z);
            glVertex3f(min_x,max_y,min_z);
        glEnd();

        glBegin(GL_LINES);
            glVertex3f(min_x,min_y,max_z);
            glVertex3f(min_x,max_y,max_z);

            glVertex3f(max_x,min_y,max_z);
            glVertex3f(max_x,max_y,max_z);

            glVertex3f(max_x,min_y,min_z);
            glVertex3f(max_x,max_y,min_z);
        glEnd();
    }

    if(DoDrawOrigin){drawOrigin();}

}

void OpenGLWidget::drawOrigin()
{
    const int ADV = 36;

    qglColor(Qt::blue);
    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(qCos(qDegreesToRadians((float)i)),qSin(qDegreesToRadians((float)i)),10);
        glVertex3f(qCos(qDegreesToRadians((float)i)),qSin(qDegreesToRadians((float)i)),0);
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(0,0,13);
        glVertex3f(2*qCos(qDegreesToRadians((float)i)),2*qSin(qDegreesToRadians((float)i)),10);
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(2*qCos(qDegreesToRadians((float)i)),2*qSin(qDegreesToRadians((float)i)),10);
        glVertex3f(0,0,10);
    }
    glEnd();

    qglColor(Qt::green);
    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(qCos(qDegreesToRadians((float)i)),0,qSin(qDegreesToRadians((float)i)));
        glVertex3f(qCos(qDegreesToRadians((float)i)),10,qSin(qDegreesToRadians((float)i)));
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(0,13,0);
        glVertex3f(2*qCos(qDegreesToRadians((float)i)),10,2*qSin(qDegreesToRadians((float)i)));
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(2*qCos(qDegreesToRadians((float)i)),10,2*qSin(qDegreesToRadians((float)i)));
        glVertex3f(0,10,0);
    }
    glEnd();

    qglColor(Qt::red);
    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(10,qCos(qDegreesToRadians((float)i)),qSin(qDegreesToRadians((float)i)));
        glVertex3f(0,qCos(qDegreesToRadians((float)i)),qSin(qDegreesToRadians((float)i)));
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(13,0,0);
        glVertex3f(10,2*qCos(qDegreesToRadians((float)i)),2*qSin(qDegreesToRadians((float)i)));
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(10,2*qCos(qDegreesToRadians((float)i)),2*qSin(qDegreesToRadians((float)i)));
        glVertex3f(10,0,0);
    }
    glEnd();


    qglColor(QColor(Qt::yellow).darker(180));
    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(1.5*qCos(qDegreesToRadians((float)i)),1.5*qSin(qDegreesToRadians((float)i)),1);
        glVertex3f(1.5*qCos(qDegreesToRadians((float)i)),1.5*qSin(qDegreesToRadians((float)i)),-1);
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(0,0,1.7);
        glVertex3f(1.5*qCos(qDegreesToRadians((float)i)),1.5*qSin(qDegreesToRadians((float)i)),1);
    }
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
    for(int i = 0; i<=360; i+=ADV){
        glVertex3f(1.5*qCos(qDegreesToRadians((float)i)),1.5*qSin(qDegreesToRadians((float)i)),-1);
        glVertex3f(0,0,-1.7);
    }
    glEnd();
}
